//
//  PlayerCentral.m
//  Auto Layout demo
//
//  The player central hosts four labeled player views with their scores
//  This abridged version adds code for auto layout expansion of the player views
//
//  Created by Bertils mini on 2010-01-21.
//  Copyright 2010-14 Sork Software. All rights reserved.
//
//  Player view base frame removed in 1.3. Nov 2014
//  Added sizeToFit to compass labels
//

#import "PlayerCentral.h"
#import "Defines.h"
#import "PlayerView.h"

@interface PlayerCentral ()
@end

@implementation PlayerCentral

// The first initializer, received before the Player View initializations
// Comes before the game controller initialization
- (id)initWithCoder:(NSCoder*)coder {
	if (self = [super initWithCoder:coder]) {
		// Create the four player views
		[self loadPlayerViews];
    }
    
	return self;
}

// Create and load four player bars, successively rotated 90 degrees CCW
- (void)loadPlayerViews {
	// First view: Place in bottom left base position for a 320 pixel square
  // Second view: Rotate and translate 90 degrees CCW ie -90 degrees
  CGAffineTransform transform1 = CGAffineTransformMakeRotation(degreesToRadians(270));
	// Third view: Rotate 180 degrees CCW
	CGAffineTransform transform2 = CGAffineTransformConcat(transform1, transform1);
	// Fourth view: Rotate 270 degrees CCW
	CGAffineTransform transform3 = CGAffineTransformConcat(transform2, transform1);
	
	// Create a PlayerView from its own nib file, the returned array is disregarded
	// Files Owner of the nib file is this Player Central
	// The playerView outlet is connected to the created view by the bundle loader
	// PlayerView tags are 0-3, the PlayerCentral tag is 4, the PlaceView tag is 5, and the lamp tag is 11
	
	for (int i = 0; i < 4; i++) {
		[[NSBundle mainBundle] loadNibNamed:@"PlayerView" owner:self options:nil];
		
		switch (i) {
			case 0:
				self.playerView.playerLabel.text = @"South 南";
				self.playerView.tag = 0;
				break;
			case 1: {
				self.playerView.transform = transform1;
				self.playerView.playerLabel.text = @"East 東";
				self.playerView.tag = 1;
				break;
			}
			case 2: {
				self.playerView.transform = transform2;
				self.playerView.playerLabel.text = @"North 北";
				self.playerView.tag = 2;
				break;
			}
			case 3: {
				self.playerView.transform = transform3;
				self.playerView.playerLabel.text = @"West 西";
				self.playerView.tag = 3;
				break;
			}
		}
    // The labels will have different lengths
    [self.playerView.playerLabel sizeToFit];

		[self addSubview:self.playerView];
	}
}

// The PlayerView corresponding to the tag (method not used here)
// As the receiver – PlayerCentral – is included in the UIView viewWithTag: search,
// the PC tag is set to 4, the PlaceView is 5, and the lampView 11
// The method viewWithTag: returns a UIView, hence the cast to PlayerView
- (PlayerView *)playerViewWithTag:(NSUInteger)tag
{
	return (PlayerView *)[self viewWithTag:tag];
}

// Add constraints to the player views that will allow these to grow on a larger screen
// The constraints are actually added later by the view controllers viewDidLoad
- (void)layoutPlayerViews
{
    // Collect the subviews involved in the auto layout
    NSMutableDictionary *views = [NSMutableDictionary new];
    // Add the central spacer view
    [views setObject:self.placeView forKey:@"Place"];
    // And the four player views
    PlayerView *aPlayer;
    
    for (aPlayer in [self subviews])
    {
        [views setObject:aPlayer forKey:[NSString stringWithFormat:@"Player%ld", (long)aPlayer.tag]];
    }
    
    // Then add the constraints for each player view
    for (aPlayer in [self subviews])
    {
        switch (aPlayer.tag) {
            case 0:
                [self addConstraints:[NSLayoutConstraint
                                      constraintsWithVisualFormat:@"V:[Place][Player0]|"
                                      options:0
                                      metrics:nil
                                      views:views]];
                [self addConstraints:[NSLayoutConstraint
                                      constraintsWithVisualFormat:@"H:|[Player0][Player1]"
                                      options:0
                                      metrics:nil
                                      views:views]];
                break;
            case 1:
                [self addConstraints:[NSLayoutConstraint
                                      constraintsWithVisualFormat:@"V:[Player2][Player1]|"
                                      options:0
                                      metrics:nil
                                      views:views]];
                [self addConstraints:[NSLayoutConstraint
                                      constraintsWithVisualFormat:@"H:[Place][Player1]|"
                                      options:0
                                      metrics:nil
                                      views:views]];
                break;
            case 2:
                [self addConstraints:[NSLayoutConstraint
                                      constraintsWithVisualFormat:@"V:|[Player2][Place]"
                                      options:0
                                      metrics:nil
                                      views:views]];
                [self addConstraints:[NSLayoutConstraint
                                      constraintsWithVisualFormat:@"H:[Player3][Player2]|"
                                      options:0
                                      metrics:nil
                                      views:views]];
                break;
            case 3:
                [self addConstraints:[NSLayoutConstraint
                                      constraintsWithVisualFormat:@"V:|[Player3][Player0]"
                                      options:0
                                      metrics:nil
                                      views:views]];
                [self addConstraints:[NSLayoutConstraint
                                      constraintsWithVisualFormat:@"H:|[Player3][Place]"
                                      options:0
                                      metrics:nil
                                      views:views]];
                break;
        }
    }
}

/* PC constraints added in IB
 "<NSLayoutConstraint:0x8e82910 H:[PlayerCentral:0x8e81ef0(320)]>",
 "<NSLayoutConstraint:0x8e82940 V:[PlayerCentral:0x8e81ef0(320)]>",
 "<NSLayoutConstraint:0x8e82970 PlayerCentral:0x8e81ef0.centerX == PlaceView:0x8e822a0.centerX>",
 "<NSLayoutConstraint:0x8e829a0 PlayerCentral:0x8e81ef0.centerY == PlaceView:0x8e822a0.centerY>",
 
   PC constraints added by layoutPlayerViews
 "<NSLayoutConstraint:0xa46f6c0 V:[PlaceView:0x8e822a0]-(0)-[PlayerView:0x8e83ff0]>",
 "<NSLayoutConstraint:0xa46f710 V:[PlayerView:0x8e83ff0]-(0)-|   (Names: '|':PlayerCentral:0x8e81ef0 )>",
 "<NSLayoutConstraint:0xa4755c0 H:|-(0)-[PlayerView:0x8e83ff0]   (Names: '|':PlayerCentral:0x8e81ef0 )>",
 "<NSLayoutConstraint:0xa474b40 H:[PlayerView:0x8e83ff0]-(0)-[PlayerView:0xa474f30]>",
 "<NSLayoutConstraint:0xa474b90 V:[PlayerView:0xa474e10]-(0)-[PlayerView:0xa474f30]>",
 "<NSLayoutConstraint:0xa46e9f0 V:[PlayerView:0xa474f30]-(0)-|   (Names: '|':PlayerCentral:0x8e81ef0 )>",
 "<NSLayoutConstraint:0xa46ea60 H:[PlaceView:0x8e822a0]-(0)-[PlayerView:0xa474f30]>",
 "<NSLayoutConstraint:0xa475970 H:[PlayerView:0xa474f30]-(0)-|   (Names: '|':PlayerCentral:0x8e81ef0 )>",
 "<NSLayoutConstraint:0xa4759a0 V:|-(0)-[PlayerView:0xa474e10]   (Names: '|':PlayerCentral:0x8e81ef0 )>",
 "<NSLayoutConstraint:0xa4759d0 V:[PlayerView:0xa474e10]-(0)-[PlaceView:0x8e822a0]>",
 "<NSLayoutConstraint:0xa478410 H:[PlayerView:0xa476f20]-(0)-[PlayerView:0xa474e10]>",
 "<NSLayoutConstraint:0xa478440 H:[PlayerView:0xa474e10]-(0)-|   (Names: '|':PlayerCentral:0x8e81ef0 )>",
 "<NSLayoutConstraint:0xa478470 V:|-(0)-[PlayerView:0xa476f20]   (Names: '|':PlayerCentral:0x8e81ef0 )>",
 "<NSLayoutConstraint:0xa4784c0 V:[PlayerView:0xa476f20]-(0)-[PlayerView:0x8e83ff0]>",
 "<NSLayoutConstraint:0xa478560 H:|-(0)-[PlayerView:0xa476f20]   (Names: '|':PlayerCentral:0x8e81ef0 )>",
 "<NSLayoutConstraint:0xa46ea20 H:[PlayerView:0xa476f20]-(0)-[PlaceView:0x8e822a0]>"
*/

@end
